@@ -72,11 +72,11 @@ static BOOL read_one_argument(char* p, string_obj* buf)
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 
     };
 
     BOOL squote = FALSE;
@@ -772,6 +772,78 @@ char* readline_method_completion(const char* text, int stat)
     return NULL;
 }
 
+static vector_obj* gEnvirons;
+
+char* readline_env_completion(const char* text, int stat_)
+{
+    static int wordlen, index;
+    static hash_it* it;
+
+    if(stat_ == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gGlobals);
+
+        index = 0;
+
+        int i;
+        for(i=0; i<vector_size(gEnvirons); i++) {
+            string_delete(vector_item(gEnvirons, i));
+        }
+        vector_clear(gEnvirons);
+
+#if !HAVE_DECL_ENVIRON
+        extern char **environ;
+#endif
+        char** p;
+        for(p = environ; *p; p++) {
+            char env_name[BUFSIZ];
+
+            char* p2 = env_name;
+            char* p3 = *p;
+
+            *p2++ = '$';
+            while(*p3 != '=' && p2-env_name < BUFSIZ-2) {
+                *p2++ = *p3++;
+            }
+
+            char* env = getenv(*p);
+            struct stat estat;
+            if(stat(env, &estat) >= 0) {
+                if(S_ISDIR(estat.st_mode)) {
+                    *p2++ = '/';
+                }
+            }
+            *p2 = 0;
+
+            vector_add(gEnvirons, STRING_NEW(env_name));
+        }
+    }
+
+    while(it) {
+        char* global = hash_loop_key(it);
+        char buf[BUFSIZ];
+        snprintf(buf, BUFSIZ, "$%s", global);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, buf, wordlen)) {
+            rl_completion_append_character= ' ';
+            return strdup(buf);
+        }
+    }
+
+    while(index < vector_size(gEnvirons)) {
+        char* env = string_c_str(vector_item(gEnvirons, index));
+
+        index++;
+
+        if(!strncmp(text, env, wordlen)) {
+            rl_completion_append_character= ' ';
+            return strdup(env);
+        }
+    }
+
+    return NULL;
+}
 
 static vector_obj* gUserCompletions;
 
@@ -821,13 +893,13 @@ char* readline_deref_completion(const char* text, int stat)
         if(!strncmp(text + 1, key, wordlen-1)) {
             if(gDerefCompletionProgram && ref->mKind == kMCObject & memchecker_is_enable_mem(ref->mMem) == kMCObject) {
                 char buf[1024];
-                snprintf(buf, 1024, "$%s->", key);
+                snprintf(buf, 1024, "`%s->", key);
                 rl_completion_append_character= 0;
                 return strdup(buf);
             }
             else {
                 char buf[1024];
-                snprintf(buf, 1024, "$%s", key);
+                snprintf(buf, 1024, "`%s", key);
                 rl_completion_append_character= ' ';
                 return strdup(buf);
             }
@@ -1100,6 +1172,7 @@ char** readline_on_complete(const char* text, int start, int end)
 {
     BOOL program = TRUE;
     BOOL user = FALSE;
+    BOOL env = FALSE;
     BOOL ref = FALSE;
     BOOL method = FALSE;
     BOOL file = FALSE;
@@ -1171,6 +1244,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1190,6 +1264,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1200,12 +1275,24 @@ char** readline_on_complete(const char* text, int start, int end)
             string_put(statment, "");
         }
         // リファレンス
-        else if(*p == '$') {
+        else if(*p == '`') {
             string_push_back2(buf, *p);
             if(program) { string_push_back2(command, *p); }
             string_push_back2(statment, *p);
             p++;
             ref = TRUE;
+            env = FALSE;
+            //program = FALSE;
+            user = FALSE;
+            method = FALSE;
+            file = FALSE;
+        }
+        // 変数
+        else if(*p == '$') {
+            string_put(buf, "");
+            p++;
+            ref = FALSE;
+            env = TRUE;
             //program = FALSE;
             user = FALSE;
             method = FALSE;
@@ -1221,6 +1308,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = FALSE;
             user = TRUE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
         }
@@ -1229,7 +1317,8 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=2;
             if(program) { 
                 sObject* object;
-                if(string_c_str(buf)[0] == '$') {
+                if(string_length(buf) > 1 && string_c_str(buf)[0] == '`') 
+                {
                     sRef* ref = saphire_get_ref(string_c_str(buf) + 1);
 
                     if(ref && ref->mKind == kMCObject && memchecker_is_enable_mem(ref->mMem) == kMCObject) {
@@ -1260,6 +1349,7 @@ char** readline_on_complete(const char* text, int start, int end)
 
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
         }
 
         /// リダイレクト
@@ -1277,6 +1367,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = FALSE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = TRUE;
 
@@ -1293,6 +1384,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = FALSE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1307,6 +1399,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1327,6 +1420,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1346,6 +1440,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1366,6 +1461,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = TRUE;
             user = FALSE;
             ref = FALSE;
+            env = FALSE;
             method = FALSE;
             file = FALSE;
 
@@ -1437,6 +1533,10 @@ char** readline_on_complete(const char* text, int start, int end)
         string_delete(statment);
         return rl_completion_matches(text, readline_deref_completion);
     }
+    else if(env) {
+        string_delete(statment);
+        return rl_completion_matches(text, readline_env_completion);
+    }
     else if(method) {
         rl_completion_append_character= ' ';
         string_delete(statment);
@@ -1819,7 +1919,7 @@ void readline_init(BOOL runtime_script)
 
     rl_attempted_completion_function = readline_on_complete;
     rl_completer_quote_characters = "\"'";
-    rl_completer_word_break_characters = " \t\n\"'|{}!&;()<>=%/";
+    rl_completer_word_break_characters = " \t\n\"'|!&;()<>=%/";
     rl_filename_quote_characters = " \\\t\n\"|$*?[]<>{}&;#()'%";
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
@@ -1850,6 +1950,8 @@ void readline_init(BOOL runtime_script)
     }
 
     files = VECTOR_NEW(30);
+    
+    gEnvirons = VECTOR_NEW(10);
 }
 
 void readline_final()
@@ -1882,6 +1984,10 @@ void readline_final()
         string_delete(str);
     }
     vector_delete(files);
+    for(i=0; i<vector_size(gEnvirons); i++) {
+        string_delete(vector_item(gEnvirons, i));
+    }
+    vector_delete(gEnvirons);
     string_delete(gEditingDir);
     string_delete(gEditingDirQuote);
 #if defined(HAVE_MIGEMO_H)
