@@ -152,6 +152,11 @@ void mmove_line_home_immediately()
 
     ttywrite(tparm(tigetstr("cub"), maxx));
 }
+/*
+static char gKeyRing[128];
+static char* gKeyRingHead;
+static char* gKeyRingTail;
+*/
 
 void mcurses_init(enum eTerminalKanjiCode code)
 {
@@ -161,7 +166,7 @@ void mcurses_init(enum eTerminalKanjiCode code)
     }
 
     gMCTtyFd = open("/dev/tty", O_RDWR);
-    char keymaps[][kKeyMapKeysMax] = {
+    signed char keymaps[][kKeyMapKeysMax] = {
         {27,   '[',   'A',   -1,   -1,  -1, -1,-1,-1,-1},   // KEY_UP
         {27,   '[',   'B',   -1,   -1,  -1, -1,-1,-1,-1},   // KYE_DOWN
         {27,   '[',   'C',   -1,   -1,  -1, -1,-1,-1,-1},   // KEY_RIGHT
@@ -224,6 +229,11 @@ void mcurses_init(enum eTerminalKanjiCode code)
     gTerminalKanjiCode = code;
     gBufferSize = 512*128;
     gBuffer = MALLOC(gBufferSize);;
+/*
+    memset(gKeyRing, 0, 128);
+    gKeyRingHead = gKeyRing;
+    gKeyRingTail = gKeyRing;
+*/
 }
 
 void mcurses_final()
@@ -332,7 +342,7 @@ void mendwin()
 
 
 
-void madd_keymap(int key, char keys[kKeyMapKeysMax])
+void madd_keymap(int key, signed char keys[kKeyMapKeysMax])
 {
     int i;
 
@@ -1899,6 +1909,94 @@ y++;
         }
     }
 }
+/*
+{
+    int n;
+    char buf[128];
+    fd_set mask;
+    struct timeval tv;
+    struct sKeyMap* keymap;
+    int count;
+    int i;
+    int j;
+
+    /// 特殊キーか判定する ///
+    if(gKeyRingHead < gKeyRingTail) {
+        for(i=0; i<vector_size(gKeyMap); i++) {
+            keymap = (struct sKeyMap*)vector_item(gKeyMap, i);
+
+            if(gKeyRingTail - gKeyRingHead >= keymap->mNum) {
+                if(memcmp(gKeyRingHead, keymap->mKeys, keymap->mNum) == 0) {
+                    gKeyRingHead += keymap->mNum;
+
+                    if(gKeyRingHead == gKeyRingTail) {
+                        gKeyRingHead = gKeyRing;
+                        gKeyRingTail = gKeyRing;
+                    }
+
+                    return keymap->mKey;
+                }
+            }
+        }
+    }
+
+    /// 読めるまで読む ///
+    count = 0;
+    while(1) {
+        FD_ZERO(&mask);
+        FD_SET(gMCTtyFd, &mask);
+
+        tv.tv_sec = 0;
+        tv.tv_usec = 0;
+
+        usleep(1);
+
+        if(select(gMCTtyFd+1, &mask, NULL, NULL, &tv) < 0) {
+            perror("select in mgetch");
+            exit(1);
+        }
+
+        if(FD_ISSET(gMCTtyFd, &mask)) {
+            if((n = ttyread(buf, 10)) < 0) {
+                perror("read in mgetch_2");
+                exit(1);
+            }
+
+            for(i=0; i<n; i++) {
+                *gKeyRingTail++ = buf[i];
+            }
+
+            /// 特殊キーか判定する ///
+            for(i=0; i<vector_size(gKeyMap); i++) {
+                keymap = (struct sKeyMap*)vector_item(gKeyMap, i);
+
+                if(gKeyRingTail - gKeyRingHead >= keymap->mNum) {
+                    if(memcmp(gKeyRingHead, keymap->mKeys, keymap->mNum) == 0) {
+                        gKeyRingHead += keymap->mNum;
+
+                        if(gKeyRingHead == gKeyRingTail) {
+                            gKeyRingHead = gKeyRing;
+                            gKeyRingTail = gKeyRing;
+                        }
+
+                        return keymap->mKey;
+                    }
+                }
+            }
+
+            /// 特殊キーじゃないなら一文字返す
+            char c = *gKeyRingHead++;
+
+            if(gKeyRingHead == gKeyRingTail) {
+                gKeyRingHead = gKeyRing;
+                gKeyRingTail = gKeyRing;
+            }
+
+            return (int)c;
+        }
+    }
+}
+*/
 
 int mgetch(int* meta)
 {
