@@ -1,13 +1,19 @@
 #include "config.h"
 #include "saphire_list.h"
+#include "saphire_vector.h"
 #include "saphire_debug.h"
 #include <stdlib.h>
 
 ///////////////////////////////////////////////////////////////////////
 // ֐
 ///////////////////////////////////////////////////////////////////////
-static list_it*
-list_it_allocate(void* item, list_it* prev_it, list_it* next_it)
+
+static list_it* list_it_release(list_it* self)
+{
+   FREE(self);
+}
+
+static list_it* list_it_allocate(void* item, list_it* prev_it, list_it* next_it)
 {
    list_it* self = (list_it*)MALLOC(sizeof(list_it));
       
@@ -18,15 +24,10 @@ list_it_allocate(void* item, list_it* prev_it, list_it* next_it)
    return self;
 }
 
-static list_it* list_it_release(list_it* self)
-{
-   FREE(self);
-}
-
 ///////////////////////////////////////////////////////////////////////
 // 
 ///////////////////////////////////////////////////////////////////////
-#if defined(MDEBUG) && !defined(HAVE_GC_H) && !defined(HAVE_GC_GC_H)
+#if defined(MDEBUG)
 
 list_obj* list_new_debug(const char* fname, int line, const char* func_name)
 {
@@ -35,24 +36,24 @@ list_obj* list_new_debug(const char* fname, int line, const char* func_name)
    self->mEntryIt = NULL;
    self->mLastIt = NULL;
    self->mCount = 0;
-   
+
    return self;
 }
 
-
 #else
+
 list_obj* list_new()
 {
-   list_obj* self = (list_obj*)MALLOC(sizeof(list_obj));
-   
-   self->mEntryIt = NULL;
-   self->mLastIt = NULL;
-   self->mCount = 0;
-   
-   return self;
+    list_obj* self = (list_obj*)MALLOC(sizeof(list_obj));
+    
+    self->mEntryIt = NULL;
+    self->mLastIt = NULL;
+    self->mCount = 0;
+ 
+    return self;
 }
 #endif
-      
+
 ///////////////////////////////////////////////////////////////////////
 // 
 ///////////////////////////////////////////////////////////////////////
@@ -276,7 +277,7 @@ void list_replace(list_it* self, void* item)
 ///////////////////////////////////////////////////////////////////////
 // Ce[^[̃ACe폜
 ///////////////////////////////////////////////////////////////////////
-void list_erase(list_it* self, list_obj* owner)
+list_it* list_erase(list_it* self, list_obj* owner)
 {
    if(self->mPrevIt) self->mPrevIt->mNextIt = self->mNextIt;
    if(self->mNextIt) self->mNextIt->mPrevIt = self->mPrevIt;
@@ -285,7 +286,10 @@ void list_erase(list_it* self, list_obj* owner)
    if(owner->mLastIt == self) owner->mLastIt = self->mPrevIt;
    owner->mCount--;
 
+   list_it* ret = self->mNextIt;
    list_it_release(self);
+
+   return ret;
 }
       
 ///////////////////////////////////////////////////////////////////////
