@@ -280,12 +280,12 @@ static BOOL read_block(char** p, string_obj* contents, vector_obj* args, string_
             string_push_back2(not_evaled, **p);
             string_push_back2(contents, *((*p)++));
         }
-        else if(**p == '{') {
+        else if(**p == '(') {
             string_push_back2(not_evaled, **p);
             string_push_back2(contents, *((*p)++));
             nest++;
         }
-        else if(**p == '}') {
+        else if(**p == ')') {
             string_push_back2(not_evaled, **p);
             (*p)++;
             nest--;
@@ -294,11 +294,11 @@ static BOOL read_block(char** p, string_obj* contents, vector_obj* args, string_
                 break;
             }
             else {
-                string_push_back2(contents, '}');
+                string_push_back2(contents, ')');
             }
         }
         else if(**p == 0) {
-            err_msg("get block: found 0. close }", sname, line_of_head);
+            err_msg("get block: found 0. close )", sname, line_of_head);
             return FALSE;
         }
         else if(**p == '\n') {
@@ -374,7 +374,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 
     };
 
     BOOL squote = FALSE;
@@ -485,7 +485,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
                 if(*(*p+1) == '$') {
                     string_push_back2(not_evaled, **p); // $
                     (*p)++;
-                    string_push_back2(not_evaled, **p); // (
+                    string_push_back2(not_evaled, **p); // $
                     (*p)++;
                 }
                 else {
@@ -500,10 +500,51 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
                 *read_end_of_statment = 1;
             }
             else {
-                string_push_back2(not_evaled, **p);
-                string_push_back2(buf, **p);
+                BOOL quote;
+                if(*(*p+1) == '$') {
+                    string_push_back2(not_evaled, **p); // $
+                    (*p)++;
+                    string_push_back2(not_evaled, **p); // $
+                    (*p)++;
 
-                (*p)++;
+                    quote = FALSE;
+                }
+                else {
+                    string_push_back2(not_evaled, **p); // $
+                    (*p)++;
+
+                    quote = TRUE;
+                }
+
+                if(**p >= '0' && **p <= '9' || **p == '{' || **p >= 'a' && **p <= 'z' || **p >= 'A' && **p <= 'Z' || **p == '_') {
+                    if(**p == '{') {
+                        if(!read_until_next_close_character('{', '}', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
+                            return FALSE;
+                        }
+                    }
+                    else {
+                        while(**p) {
+                            if(**p >= '0' && **p <= '9' || **p >= 'a' && **p <= 'z' || **p >= 'A' && **p <= 'Z' || **p == '_') {
+                                string_push_back2(not_evaled, **p); // 名前
+                                (*p)++;
+                            }
+                            else {
+                                break;
+                            }
+                        }
+                    }
+
+                    *read_end_of_statment = 1;
+                }
+                else {
+                    if(quote) {
+                        string_push_back2(buf, '$');
+                    }
+                    else {
+                        string_push_back2(buf, '$');
+                        string_push_back2(buf, '$');
+                    }
+                }
             }
         }
         /// シングルクォート／ダブルクォート中
@@ -621,7 +662,8 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             }
         }
         /// グロブ ///
-        else if(command && vector_size(command->mArgs) > 0 && (**p == '[' || **p == '?' || **p == '*')) {
+        else if(command && vector_size(command->mArgs) > 0 && (**p == '[' || **p == '?' || **p == '*')) 
+        {
             string_push_back2(not_evaled, **p);
             string_push_back2(buf, **p);
             (*p)++;
@@ -657,7 +699,8 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             }
         }
         // ->は下で処理
-        else if(command && vector_size(command->mArgs) == 0 && **p == '-' && *(*p+1) == '>') {
+        else if(command && vector_size(command->mArgs) == 0 && **p == '-' && *(*p+1) == '>') 
+        {
             string_push_back2(not_evaled, **p);
             string_push_back2(buf, **p);
 
@@ -676,10 +719,11 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             (*p)++;
         }
         // クィックコマンド展開は下で処理
-        else if((**p == '@' && *(*p+1) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) >= 'a' && *(*p+2) <= 'z' && *(*p+3) == '(')) {
+        else if(**p == '@' && (*(*p+1) == '(') || (*(*p+1) == '@' && *(*p+2) == '(') || (*(*p+1) == '@' && *(*p+2) >= 'a' && *(*p+2) <= 'z' && *(*p+3) == '(')) 
+        {
             break;
         }
-        // % のりダイレクトは下で処理
+        // % のリダイレクトは下で処理
         else if(**p == '%' && (*(*p+1) == '>' || *(*p+1) == '2')) {
             break;
         }
@@ -744,7 +788,7 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
     /// 実行文 ///
     sStatments* loop = STATMENTS_NEW();
 
-    if(**p == '{') {
+    if(**p == '(') {
         string_obj* str = STRING_NEW("");
         if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
             string_delete(str);
@@ -855,7 +899,7 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
             /// 実行文 ///
             sStatments* loop = STATMENTS_NEW();
 
-            if(**p == '{') {
+            if(**p == '(') {
                 string_obj* str = STRING_NEW("");
                 if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
                     string_delete(str);
@@ -927,7 +971,7 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
         /// 実行文 ///
         sStatments* loop = STATMENTS_NEW();
 
-        if(**p == '{') {
+        if(**p == '(') {
             string_obj* str = STRING_NEW("");
             if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
                 string_delete(str);
@@ -1034,7 +1078,7 @@ static BOOL read_command_parse_while_statment(char** p, sCommand* command, sStat
     /// ループ文読み込み ///
     sStatments* loops = STATMENTS_NEW();
 
-    if(**p == '{') {
+    if(**p == '(') {
         string_obj* str = STRING_NEW("");
         if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
             sStatments_delete(conditions);
@@ -1504,8 +1548,9 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
         else if(**p == ' ' || **p == '\t') {
             skip_spaces(p, not_evaled);
         }
-        /// サブシェル ///
+        /// サブシェル、ブロック ///
         else if(**p == '(') {
+            /// コマンド名の所にあればサブシェル ///
             if(vector_size(command->mArgs) == 0) {
                 string_obj* str = STRING_NEW("");
                 int read_end_of_statment = -1;
@@ -1533,57 +1578,51 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                 vector_add(command->mArgs, ARG_NEW(0, MANAGED STRING_NEW("subshell")));
                 command->mExtra = sSubshell_new(MANAGED statments);
             }
+            /// ブロック ///
             else {
-                char buf[128];
-                snprintf(buf, 128, "unexpected token2 -->%c\n", **p);
-                err_msg(buf, sname, *sline);
-                return FALSE;
-            }
-        }
-        /// ブロック ///
-        else if(**p == '{') {
-            string_obj* str = STRING_NEW("");
-            vector_obj* args = VECTOR_NEW(10);
-            if(!read_block(p, str, args, not_evaled, FALSE, sname, sline)) {
+                string_obj* str = STRING_NEW("");
+                vector_obj* args = VECTOR_NEW(10);
+                if(!read_block(p, str, args, not_evaled, FALSE, sname, sline)) {
+                    int i;
+                    for(i=0; i<vector_size(args); i++) {
+                        string_delete(vector_item(args, i));
+                    }
+                    vector_delete(args);
+                    string_delete(str);
+                    return FALSE;
+                }
+
+                sStatments* block = STATMENTS_NEW();
+                if(!parse(string_c_str(str), sname, sline, block))
+                {
+                    sStatments_delete(block);
+                    int i;
+                    for(i=0; i<vector_size(args); i++) {
+                        string_delete(vector_item(args, i));
+                    }
+                    vector_delete(args);
+                    string_delete(str);
+                    return FALSE;
+                }
+
+                int count = 0;
                 int i;
-                for(i=0; i<vector_size(args); i++) {
-                    string_delete(vector_item(args, i));
+                for(i=0; i<vector_size(command->mArgs); i++) {
+                    sArg* arg = vector_item(command->mArgs, i);
+                    if(arg->mKind != 5) {
+                        count++;
+                    }
                 }
-                vector_delete(args);
+
+                vector_add(command->mArgs, ARG_NEW(5, MANAGED sBlock_new(MANAGED block, MANAGED args, count-1)));
+
+                string_push_back(statment->mTitle, string_c_str(str));
+                string_push_back(statment->mTitle, " ");
+
+                skip_spaces(p, not_evaled);
+
                 string_delete(str);
-                return FALSE;
             }
-
-            sStatments* block = STATMENTS_NEW();
-            if(!parse(string_c_str(str), sname, sline, block))
-            {
-                sStatments_delete(block);
-                int i;
-                for(i=0; i<vector_size(args); i++) {
-                    string_delete(vector_item(args, i));
-                }
-                vector_delete(args);
-                string_delete(str);
-                return FALSE;
-            }
-
-            int count = 0;
-            int i;
-            for(i=0; i<vector_size(command->mArgs); i++) {
-                sArg* arg = vector_item(command->mArgs, i);
-                if(arg->mKind != 5) {
-                    count++;
-                }
-            }
-
-            vector_add(command->mArgs, ARG_NEW(5, MANAGED sBlock_new(MANAGED block, MANAGED args, count-1)));
-
-            string_push_back(statment->mTitle, string_c_str(str));
-            string_push_back(statment->mTitle, " ");
-
-            skip_spaces(p, not_evaled);
-
-            string_delete(str);
         }
         else {
             break;
@@ -1963,8 +2002,136 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
 
                 sWFd_delete(pipeout);
             }
+            /// var, global, export 展開 ///
             else {
-                string_push_back2(out, *p++);
+                BOOL quote;
+                if(*(p+1) == '$') {
+                    p+=2;
+                    quote = FALSE;
+                }
+                else {
+                    p++;
+                    quote = TRUE;
+                }
+
+                if((*p >= '0' && *p <= '9' || *p >= 'a' && *p <= 'z' || *p >= 'A' && *p <= 'Z' || *p == '_') || *p == '{') {
+                    string_obj* name = STRING_NEW("");
+
+                    BOOL bracket;
+                    if(*p == '{') {
+                        int read_end_of_statment = -1;
+                        if(!read_until_next_close_character('{', '}',&p, name, &read_end_of_statment, TRUE, sname, sline)) {
+                            string_delete(name);
+                            return FALSE;
+                        }
+
+                        string_erase(name, 0, 1);
+                        string_trunc(name, string_length(name)-1);
+                        bracket = TRUE;
+                    }
+                    else {
+                        bracket = FALSE;
+                        while(*p) {
+                            if(*p >= '0' && *p <= '9' || *p >= 'a' && *p <= 'z' || *p >= 'A' && *p <= 'Z' || *p == '_') {
+                                string_push_back2(name, *p); // $
+                                p++;
+                            }
+                            else {
+                                break;
+                            }
+                        }
+                    }
+
+                    char* name2 = string_c_str(name);
+
+                    /// Var ///
+                    hash_obj* top_stack
+                         = vector_item(gStackFrame, vector_size(gStackFrame)-1);
+                    string_obj* var = hash_item(top_stack, name2);
+
+                    if(var) {
+                        if(quote) {
+                            string_obj* quoted = STRING_NEW("");
+                            if(!expand_env_get_quoted_str(string_c_str(var), quoted, sname, sline))
+                            {
+                                string_delete(name);
+                                return FALSE;
+                            }
+                            string_push_back(out, string_c_str(quoted));
+                            string_delete(quoted);
+                        }
+                        else {
+                            string_push_back(out, string_c_str(var));
+                        }
+                    }
+                    else {
+                        /// Global ///
+                        if(object) {
+                            var = hash_item(object->mGlobals, name2);
+
+                            if(var == NULL) {
+                                sObject* parent = object->mParent;
+                                while(parent) {
+                                    var = hash_item(parent->mGlobals, name2);
+                                    if(var) break;
+                                    parent = parent->mParent;
+                                }
+                                if(var == NULL) {
+                                    var = hash_item(gGlobals, name2);
+                                }
+                            }
+                        } else {
+                            var = hash_item(gGlobals, name2);
+                        }
+
+                        if(var) {
+                            if(quote) {
+                                string_obj* quoted = STRING_NEW("");
+                                if(!expand_env_get_quoted_str(string_c_str(var), quoted, sname, sline))
+                                {
+                                    string_delete(name);
+                                    return FALSE;
+                                }
+                                string_push_back(out, string_c_str(quoted));
+                                string_delete(quoted);
+                            }
+                            else {
+                                string_push_back(out, string_c_str(var));
+                            }
+                        }
+                        else {
+                            /// export ///
+                            char* env = getenv(name2);
+                            if(env) {
+                                if(quote) {
+                                    string_obj* quoted = STRING_NEW("");
+                                    if(!expand_env_get_quoted_str(env, quoted, sname, sline))
+                                    {
+                                        string_delete(name);
+                                        return FALSE;
+                                    }
+                                    string_push_back(out, string_c_str(quoted));
+                                    string_delete(quoted);
+                                }
+                                else {
+                                    string_push_back(out, env);
+                                }
+                            }
+                            else {
+                            }
+                        }
+                    }
+
+                    string_delete(name);
+                }
+                else {
+                    if(quote) {
+                        string_push_back2(out, '$');
+                    }
+                    else {
+                        string_push_back(out, "$$");
+                    }
+                }
             }
         }
         else if(squote || dquote) {
@@ -1976,16 +2143,7 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
             string_push_back2(out, *p++);
         }
 
-        /// ブロックは飛ばす
-        else if(*p == '{') {
-            string_obj* dummy = STRING_NEW("");
-            if(!read_block(&p, dummy, NULL, out, FALSE, sname, sline)) {
-                string_delete(dummy);
-                return FALSE;
-            }
-            string_delete(dummy);
-        }
-        /// サブシェル、条件式は飛ばす
+        /// ブロック、サブシェル、条件式は飛ばす
         else if(*p == '(') {
             int read_end_of_statment = -1;
             if(!read_until_next_close_character('(', ')', &p, out, &read_end_of_statment, FALSE, sname, sline))
@@ -2001,16 +2159,252 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
     return TRUE;
 }
 
+BOOL expand_brace_expression(string_obj* tail, vector_obj* v, string_obj* start, string_obj* end, char** p, char* sname, int* sline)
+{
+    string_obj* str = STRING_NEW("");
+
+    while(1) {
+        if(**p == '\\') {
+            (*p)++;
+            string_push_back2(str, '\\');
+            string_push_back2(str, **p);
+            (*p)++;
+        }
+        else if(**p == '{') {
+            (*p)++;
+
+            vector_obj* v2 = VECTOR_NEW(10);
+            string_obj* tail2 = STRING_NEW("");
+            string_obj* start2 = STRING_NEW("");
+            string_obj* end2 = STRING_NEW("");
+            if(!expand_brace_expression(tail2, v2, start2, end2, p, sname, sline)) {
+                string_delete(str);
+                string_delete(tail2);
+                string_delete(start2);
+                string_delete(end2);
+                int i;
+                for(i=0; i<vector_size(v2); i++) {
+                    string_delete(vector_item(v2, i));
+                }
+                vector_delete(v2);
+                return FALSE;
+            }
+
+            if(**p == ',') { (*p)++; }
+
+            /// string
+            if(string_c_str(start2)[0] == 0) {
+                if(vector_size(v2) == 0) {
+                    err_msg("invalid brace expression2", sname, *sline);
+                    string_delete(str);
+                    string_delete(tail2);
+                    vector_delete(v2);
+                    string_delete(start2);
+                    string_delete(end2);
+                    return FALSE;
+                }
+
+                int i;
+                for(i=0; i<vector_size(v2); i++) {
+                    string_obj* str2 = STRING_NEW("");
+                    string_push_back(str2, string_c_str(str));
+                    string_push_back(str2, string_c_str(vector_item(v2, i)));
+                    string_push_back(str2, string_c_str(tail2));
+
+                    vector_add(v, str2);
+                }
+            }
+            /// number
+            else {
+                int start3 = atoi(string_c_str(start2));
+                int end3 = atoi(string_c_str(end2));
+
+                int num_len = strlen(string_c_str(start2));
+                if(start3 < 0) num_len--;
+
+                char buf2[1024];
+                buf2[0] = '%';
+                snprintf(buf2 + 1, 1023, "0%dd", num_len);
+
+                if(start3 < end3) {
+                    int i;
+                    for(i=start3; i<=end3; i++) {
+                        string_obj* str2 = STRING_NEW("");
+
+                        string_push_back(str2, string_c_str(str));
+
+                        char buf[1024];
+                        if(i < 0) {
+                            buf[0] = '-';
+                            snprintf(buf + 1, 1024, buf2, abs(i));
+                        }
+                        else {
+                            snprintf(buf, 1024, buf2, i);
+                        }
+
+                        string_push_back(str2, buf);
+
+                        string_push_back(str2, string_c_str(tail2));
+
+                        vector_add(v, str2);
+                    }
+                }
+                else if(start3 == end3) {
+                    vector_add(v, STRING_NEW(string_c_str(start2)));
+                }
+                else {
+                    int i;
+                    for(i=start3; i>=end3; i--) {
+                        string_obj* str2 = STRING_NEW("");
+
+                        string_push_back(str2, string_c_str(str));
+
+                        char buf[1024];
+                        if(i < 0) {
+                            buf[0] = '-';
+                            snprintf(buf + 1, 1024, buf2, abs(i));
+                        }
+                        else {
+                            snprintf(buf, 1024, buf2, i);
+                        }
+                        string_push_back(str2, buf);
+
+                        string_push_back(str2, string_c_str(tail2));
+
+                        vector_add(v, str2);
+                    }
+                }
+            }
+
+            string_put(str, "");
+            string_delete(tail2);
+            int i;
+            for(i=0; i<vector_size(v2); i++) {
+                string_delete(vector_item(v2, i));
+            }
+            vector_delete(v2);
+
+            string_delete(start2);
+            string_delete(end2);
+        }
+        else if(**p == '}') {
+            (*p)++;
+
+            if(*(*p-2) == ',') {
+                    vector_add(v, str);
+            }
+            else {
+                if(string_c_str(str)[0] == 0) {
+                    string_delete(str);
+                }
+                else {
+                    vector_add(v, str);
+                }
+            }
+
+            while(**p) {
+                if(**p == '\\') {
+                    (*p)++;
+                    string_push_back2(tail, '\\');
+                    string_push_back2(tail, **p);
+                    (*p)++;
+                }
+                else if(**p == ' ' || **p == '\t' || **p == '\n' || **p == ',' || **p=='>' || **p=='}'  || **p=='&' || **p=='~' || **p=='#' || **p =='$' || **p=='(' || **p==')' || **p=='|' || **p==';' || **p=='\'' || **p=='"' || **p=='<' || **p=='%' || **p=='@')
+                {
+                    break;
+                }
+                else {
+                    string_push_back2(tail, **p);
+                    (*p)++;
+                }
+            }
+
+            break;
+        }
+        else if(**p == ',') {
+            (*p)++;
+
+            vector_add(v, str);
+            str = STRING_NEW("");
+        }
+        else if(**p == '.' && *(*p+1) == '.') {
+            (*p)+=2;
+
+            string_put(start, string_c_str(str));
+
+            string_put(str, "");
+
+            if(**p == '-') {
+                string_push_back2(str, **p);
+                (*p)++;
+            }
+
+            while(1) {
+                if(**p == '}') {
+                    (*p)++;
+                    break;
+                }
+                else if(**p >= '0' && **p <= '9') {
+                    string_push_back2(str, **p);
+                    (*p)++;
+                }
+                else {
+                    err_msg("invalid brace expression", sname, *sline);
+                    string_delete(str);
+                    return FALSE;
+                }
+            }
+
+            string_put(end, string_c_str(str));
+
+            while(**p) {
+                if(**p == '\\') {
+                    (*p)++;
+                    string_push_back2(tail, '\\');
+                    string_push_back2(tail, **p);
+                    (*p)++;
+                }
+                else if(**p == ' ' || **p == '\t' || **p == '\n' || **p == ',' || **p=='}' || **p=='>' || **p=='&' || **p=='~' || **p=='#' || **p =='$' || **p=='(' || **p==')' || **p=='|' || **p==';' || **p=='\'' || **p=='"' || **p=='<' || **p=='%' || **p=='@')
+                {
+                    break;
+                }
+                else {
+                    string_push_back2(tail, **p);
+                    (*p)++;
+                }
+            }
+
+            string_delete(str);
+            break;
+        }
+        else if(**p == 0) {
+            err_msg("invalid brace expression", sname, *sline);
+            string_delete(str);
+            return FALSE;
+        }
+        else {
+            string_push_back2(str, **p);
+            (*p)++;
+        }
+    }
+
+    return TRUE;
+}
+
 // コメントも除去する
 // 読み込みだけ。この後パースする
+// ブレース展開を行う
 BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname, int* sline)
 {
     BOOL squote = FALSE;
     BOOL dquote = FALSE;
 
+    string_obj* head = STRING_NEW("");
     while(*p) {
         if(!dquote && !squote && *p == '\\') {
             string_push_back2(cmdline2, *p++);
+
+            string_push_back2(head, *p);
             string_push_back2(cmdline2, *p++);
         }
         else if(!dquote && *p == '\'') {
@@ -2024,12 +2418,21 @@ BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname
         else if(squote || dquote) {
             if(*p == 0) {
                 err_msg("close \" or \' before the end of comments", sname, *sline);
+                string_delete(head);
                 return FALSE;
             }
             else {
+                string_push_back2(head, *p);
                 string_push_back2(cmdline2, *p++);
             }
         }
+        /// 変数は虫
+        else if(*p == '$' && *(p+1) == '{') {
+            string_push_back2(head, *p);
+            string_push_back2(cmdline2, *p++);
+            string_push_back2(head, *p);
+            string_push_back2(cmdline2, *p++);
+        }
         /// コメント ///
         else if(*p == '#') {
             p++;
@@ -2046,11 +2449,139 @@ BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname
                 }
             }
         }
+        /// ブレース展開
+        else if(*p == '{') {
+            p++;
+
+            vector_obj* v = VECTOR_NEW(10);
+            string_obj* tail = STRING_NEW("");
+            string_obj* start = STRING_NEW("");
+            string_obj* end = STRING_NEW("");
+            if(!expand_brace_expression(tail, v, start, end, &p, sname, sline)) {
+                string_delete(head);
+                string_delete(tail);
+                string_delete(start);
+                string_delete(end);
+                int i;
+                for(i=0; i<vector_size(v); i++) {
+                    string_delete(vector_item(v, i));
+                }
+                vector_delete(v);
+                return FALSE;
+            }
+
+            /// string
+            if(string_c_str(start)[0] == 0) {
+                if(vector_size(v) == 0) {
+                    err_msg("invalid brace expression", sname, *sline);
+                    string_delete(head);
+                    string_delete(tail);
+                    string_delete(start);
+                    string_delete(end);
+                    vector_delete(v);
+                    return FALSE;
+                }
+
+                string_push_back(cmdline2, string_c_str(vector_item(v, 0)));
+                string_push_back(cmdline2, string_c_str(tail));
+                string_push_back2(cmdline2, ' ');
+
+                int i;
+                for(i=1; i<vector_size(v); i++) {
+                    string_push_back(cmdline2, string_c_str(head));
+                    string_push_back(cmdline2, string_c_str(vector_item(v, i)));
+                    string_push_back(cmdline2, string_c_str(tail));
+                    if(i < vector_size(v)-1) string_push_back2(cmdline2, ' ');
+                }
+            }
+            /// number
+            else {
+                int start2 = atoi(string_c_str(start));
+                int end2 = atoi(string_c_str(end));
+
+                int num_len = strlen(string_c_str(start));
+                if(num_len < 0) num_len--;
+
+                char buf2[1024];
+                buf2[0] = '%';
+                snprintf(buf2 + 1, 1023, "0%dd", num_len);
+
+                if(start2 < end2) {
+                    string_push_back(cmdline2, string_c_str(start));
+                    string_push_back(cmdline2, string_c_str(tail));
+                    string_push_back2(cmdline2, ' ');
+
+                    int i;
+                    for(i=start2+1; i<=end2; i++) {
+                        string_push_back(cmdline2, string_c_str(head));
+
+                        char buf[1024];
+                        if(i < 0) {
+                            buf[0] = '-';
+                            snprintf(buf + 1, 1024, buf2, abs(i));
+                        }
+                        else {
+                            snprintf(buf, 1024, buf2, i);
+                        }
+
+                        string_push_back(cmdline2, buf);
+                        string_push_back(cmdline2, string_c_str(tail));
+                        if(i <= end2-1) string_push_back2(cmdline2, ' ');
+                    }
+                }
+                else if(start2 == end2) {
+                    string_push_back(cmdline2, string_c_str(start));
+                    string_push_back(cmdline2, string_c_str(tail));
+                    string_push_back2(cmdline2, ' ');
+                }
+                else {
+                    string_push_back(cmdline2, string_c_str(start));
+                    string_push_back(cmdline2, string_c_str(tail));
+                    string_push_back2(cmdline2, ' ');
+
+                    int i;
+                    for(i=start2-1; i>=end2; i--) {
+                        string_push_back(cmdline2, string_c_str(head));
+
+                        char buf[1024];
+                        if(i < 0) {
+                            buf[0] = '-';
+                            snprintf(buf + 1, 1024, buf2, abs(i));
+                        }
+                        else {
+                            snprintf(buf, 1024, buf2, i);
+                        }
+                        string_push_back(cmdline2, buf);
+
+                        string_push_back(cmdline2, string_c_str(tail));
+
+                        if(i>= end2 + 1) string_push_back2(cmdline2, ' ');
+                    }
+                }
+            }
+
+            string_put(head, "");
+            string_delete(tail);
+            int i;
+            for(i=0; i<vector_size(v); i++) {
+                string_delete(vector_item(v, i));
+            }
+            vector_delete(v);
+            string_delete(start);
+            string_delete(end);
+        }
+        else if(*p == ' ' || *p == '\t' || *p == '\n') {
+            string_push_back2(cmdline2, *p++);
+
+            string_put(head, "");
+        }
         else {
+            string_push_back2(head, *p);
             string_push_back2(cmdline2, *p++);
         }
     }
 
+    string_delete(head);
     return TRUE;
 }
 
