@@ -5,8 +5,6 @@
 #include <stdio.h>
 #include <string.h>
 
-extern BOOL gKitutukiSigInt;  // CTRL-Cが押されたらTRUE
-
 //////////////////////////////////////////////////////////////////////
 // 生成
 //////////////////////////////////////////////////////////////////////
@@ -162,9 +160,6 @@ static BOOL quick_sort(vector_obj* self, int left, int right, sort_if fun)
                     break;
                 }
                 i++;
-                if(gKitutukiSigInt) {
-                    return FALSE;
-                }
             }
                      
             while(1) {
@@ -175,9 +170,6 @@ static BOOL quick_sort(vector_obj* self, int left, int right, sort_if fun)
                     break;
                 }
                 j--;
-                if(gKitutukiSigInt) {
-                    return FALSE;
-                }
             }
 
             if(i <= j) {
@@ -188,9 +180,6 @@ static BOOL quick_sort(vector_obj* self, int left, int right, sort_if fun)
                 i++;
                 j--;
             }
-            if(gKitutukiSigInt) {
-                return FALSE;
-            }
         } while(i <= j);
 
         if(!quick_sort(self, left, j, fun)) {
@@ -204,14 +193,9 @@ static BOOL quick_sort(vector_obj* self, int left, int right, sort_if fun)
     return TRUE;
 }
 
-BOOL vector_sort(vector_obj* self, sort_if fun)
+void vector_sort(vector_obj* self, sort_if fun)
 {
-    if(!quick_sort(self, 0, self->mCount-1, fun)) {
-        gKitutukiSigInt = FALSE;
-        return FALSE;
-    }
-
-    return TRUE;
+    quick_sort(self, 0, self->mCount-1, fun);
 }
 
 //////////////////////////////////////////////////////////////////////
